#ifndef _TEXTURE_H_
#define _TEXTURE_H_

#include <fstream.h>
#include <windows.h>
#include "vector.h"

class Texture {
    unsigned char *buf;
    long size, size_x, size_y, u, v;
    double scale;
    Vector GetPixel(long x, long y);
  public:
    Texture(char *, int, int, double);
    ~Texture();
    Vector GetColor(double, double);
};

Texture::Texture(char *filename, int u, int v, double scale)
{
  BITMAPINFOHEADER info;
  BITMAPFILEHEADER head;
  Texture::u = u;
  Texture::v = v;
  Texture::scale = scale;
  buf = NULL;
  ifstream in(filename, ios::binary);
  if (!in) return;
  in.read((char *)&head, sizeof(head));
  in.read((char *)&info, sizeof(info));
  if (info.biBitCount != 24) return;
  size_x = info.biWidth;
  size_y = info.biHeight;
  size = size_x * size_y * 3;
  buf = new char[size];
  in.seekg(head.bfOffBits, ios::beg);
  in.seekg(0x36, ios::beg);
  in.read(buf, size);
  in.close();
}

Texture::~Texture()
{
  if (buf != NULL)
    delete buf;
}

Vector Texture::GetPixel(long x, long y)
{
  long tx = x % size_x;
  long ty = y % size_y;
  if (tx < 0) tx += size_x;
  if (ty < 0) ty += size_y;
  long i = (ty * size_x + tx) * 3;
  return Vector(double(buf[i + 2]) / 255, double(buf[i + 1]) / 255, double(buf[i]) / 255);
}

Vector Texture::GetColor(double x, double y)
{
  double d1 = x / scale + u, d2 = y / scale + v;
  double t1 = d1 - long(d1), t2 = d2 - long(d2);
  Vector p1 = GetPixel(d1, d2), p2 = GetPixel(d1, d2 + 1);
  Vector p3 = GetPixel(d1 + 1, d2), p4 = GetPixel(d1 + 1, d2 + 1);
  Vector c1(p1), c2(p3);
  if (t1 < 0) t1++;
  if (t2 < 0) t2++;
  c1 += (p2 - p1) * t2;
  c2 += (p4 - p3) * t2;
  return c1 + (c2 - c1) * t1;
}

#endif