#ifndef _VECTOR_H_
#define _VECTOR_H_
#include <math.h>

class Vector {
  public:
    double x, y, z;
    Vector() { }
    Vector(double x, double y, double z)
    {
      Vector::x = x;
      Vector::y = y;
      Vector::z = z;
    }
    Vector(Vector& v)
    {
      x = v.x;
      y = v.y;
      z = v.z;
    }
    Vector(double v)
    {
      x = y = z = v;
    }
    Vector& operator =(Vector& v)
    {
      x = v.x;
      y = v.y;
      z = v.z;
      return *this;
    }
    Vector& operator =(double d)
    {
      x = y = z = d;
      return *this;
    }
    Vector& operator +=(Vector& v)
    {
      x += v.x;
      y += v.y;
      z += v.z;
      return *this;
    }
    Vector& operator -=(Vector& v)
    {
      x -= v.x;
      y -= v.y;
      z -= v.z;
      return *this;
    }
    Vector operator -()
    {
      return Vector(-x, -y, -z);
    }
    Vector& operator *=(Vector& v)
    {
      x *= v.x;
      y *= v.y;
      z *= v.z;
      return *this;
    }
    Vector& operator *=(double d)
    {
      x *= d;
      y *= d;
      z *= d;
      return *this;
    }
    Vector& operator /=(Vector& v)
    {
      x /= v.x;
      y /= v.y;
      z /= v.z;
      return *this;
    }
    Vector& operator /=(double d)
    {
      x /= d;
      y /= d;
      z /= d;
      return *this;
    }
    friend Vector operator +(Vector& v1, Vector& v2)
    {
      return Vector(v1.x + v2.x, v1.y + v2.y, v1.z + v2.z);
    }
    friend Vector operator -(Vector& v1, Vector& v2)
    {
      return Vector(v1.x - v2.x, v1.y - v2.y, v1.z - v2.z);
    }
    friend Vector operator *(Vector& u, Vector& v)
    {
      return Vector(u.x * v.x, u.y * v.y, u.z * v.z);
    }
    friend Vector operator *(Vector& v, double d)
    {
      return Vector(v.x * d, v.y * d, v.z * d);
    }
    friend Vector operator *(double d, Vector& v)
    {
      return Vector(v.x * d, v.y * d, v.z * d);
    }
    friend Vector operator /(Vector& u, Vector& v)
    {
      return Vector(u.x / v.x, u.y / v.y, u.z / v.z);
    }
    friend Vector operator /(Vector& v, double d)
    {
      return Vector(v.x / d, v.y / d, v.z / d);
    }
    double& operator [](int n)
    {
      return *(&x + n);
    }
    double operator !()
    {
      return sqrt(x * x + y * y + z * z);
    }
    friend double operator &(Vector& u, Vector& v)
    {
      return u.x * v.x + u.y * v.y + u.z * v.z;
    }
    friend Vector operator ^(Vector& u, Vector& v)
    {
      return Vector(u.y * v.z - u.z * v.y,
                    u.z * v.x - u.x * v.z,
                    u.x * v.y - u.y * v.x);
    }
};

Vector Normalize(Vector v)
{
  return v / !v;
}

Vector Clip(Vector v)
{
  Vector res(v);

  if (res.x > 1.0)
    res.x = 1.0;
  else if (res.x < 0.0)
    res.x = 0.0;

  if (res.y > 1.0)
    res.y = 1.0;
  else if (res.y < 0.0)
    res.y = 0.0;

  if (res.z > 1.0)
    res.z = 1.0;
  else if (res.z < 0.0)
    res.z = 0.0;

  return res;
}

#endif